// kate: space-indent on; indent-width 2; replace-tabs on;
/***********************************************************************
 *
 *   18-8-95  Created by Jean-Claude Colson.
 *
 *   RISK game player.
 *   $Id: aiColson.c,v 1.5 1999/11/07 15:57:29 tony Exp $
 *
 ***********************************************************************/

#include "aiColsonPlayer.h"

#include "onu.h"
#include "goal.h"
#include "continent.h"
#include "gameautomaton.h"
#include "kgamewin.h"
#include "xfriskaiclient.h"
#include "aiplayerio.h"

#include <assert.h>
#include <limits>

namespace Ksirk
{

namespace GameLogic
{


AIColsonPlayer::AIColsonPlayer(
        const QString & nomPlayer, unsigned int nbArmies,
        Nationality * myNation,  PlayersArray& players, ONU* world,
        GameAutomaton* game ) :
  AIPlayer(nomPlayer, nbArmies, myNation, players, world, game ),
  m_levelEnemy(0),
  m_initialized(false),
  Attack_SrcCountry(-1),
  Attack_DestCountry(-1),
  m_placeData(0)
{
  kdDebug() << "AIColsonPlayer constructor" << endl;
}

AIColsonPlayer::~AIColsonPlayer() 
{
}

//////////////////////////////////////////////////////////
// Virtual functions reimplemented from AIPlayer
//////////////////////////////////////////////////////////

std::pair< const Country*, const Country* > AIColsonPlayer::chooseBelligerant()
{
  kdDebug() << "AIColsonPlayer::chooseBelligerant" << endl;
  Country* src = 0;
  Country* dest = 0;
  
//   Attack_SrcCountry = -1;
//   Attack_DestCountry = -1;

  // no attack was tempted
  if (!Attack())
  {
    Attack_SrcCountry = -1;
    Attack_DestCountry = -1;
    return std::make_pair(static_cast<Country*>(0), static_cast<Country*>(0));
  }

  for (int i=0; i<m_world->getCountries().count(); i++)
  {
    if (i == Attack_SrcCountry)
    {
      src = m_world->getCountries().at(i);
    }
    if (i == Attack_DestCountry)
    {
      dest = m_world->getCountries().at(i);
    }
  }
  kdDebug() << "choosed belligerants " << src << " and " << dest << endl;
  return std::make_pair(src,dest);
}

/**
  * Chooses the next action. Attack by default and if no attack is possible,
  * try to move armies and in the last resort, choose next player
  */
void AIColsonPlayer::chooseAttackMoveArmiesOrNextPlayer()
{
  kdDebug() << "AIColsonPlayer::chooseAttackMoveArmiesOrNextPlayer() " << endl;
  if (!m_initialized)
  {
    finalize();
  }

  if (!attackAction())
  {
    if (!moveArmiesAction())
    {
      nextPlayerAction();
    }
  }
//    kdDebug() <<"OUT AIColsonPlayer::chooseAttackMoveArmiesOrNextPlayer()" << endl;
}

/**
  * Chooses a country to receive a new army in dotation
  */
Country* AIColsonPlayer::chooseReceivingCountry()
{
  kdDebug() << "AIColsonPlayer::chooseReceivingCountry" << endl;
  if (m_placeData == 0)
  {
    if (!Place())
    {
      return 0;
    }
  }
  Country* res = m_placeData->dest;
  m_placeData->nb--;
  if (m_placeData->nb == 0)
  {
    delete m_placeData;
    m_placeData = 0;
  }
  return res;
}

/**
  * makes all what is necessary to prepare and start the moving of armies
  */
bool AIColsonPlayer::moveArmiesAction() 
{
  kdDebug() << "AIColsonPlayer::moveArmiesAction" << endl;
  bool res = Move();
  kdDebug() << "AIColsonPlayer::moveArmiesAction Move got " << res << endl;
  return res;
}

//////////////////////////////////////////////////////////
// XFrisk AIColson functions
//////////////////////////////////////////////////////////
int AIColsonPlayer::getTotalArmiesOfPlayer(const Player* player)
{
  int nb = 0;
  for (int i=0; i<m_world->getCountries().count(); i++)
  {
    if (m_world->getCountries().at(i)->owner() == player)
    {
      nb += m_world->getCountries().at(i)->nbArmies();
    }
  }
  return nb;
}


bool AIColsonPlayer::isContinentOfMission(const Player* player, const Continent* continent)
{
  if (player->goal()->type() != Goal::Continents)
      return false;
  return (player->goal()->continents().find(continent->id()) != player->goal()->continents().end());
}

bool AIColsonPlayer::isEnemyPlayer(const Player* player)
{
  return m_isEnemyPlayer[player];
}

bool AIColsonPlayer::isFriendPlayer(const Player* player)
{
  return m_isEnemyPlayer[player] < m_levelEnemy;
}

int AIColsonPlayer::getNumEnemy()
{
  kdDebug() << "AIColsonPlayer::getNumEnemy" << endl;
  int nb = 0;

  PlayersArray::iterator it = m_game->playerList()->begin();
  PlayersArray::iterator it_end = m_game->playerList()->end();
  for (; it != it_end; it++)
  {
    if (m_isEnemyPlayer[((Player*)(*it))]>= m_levelEnemy)
    {
      nb++;
    }
  }
  if (m_levelEnemy == 1)
      nb--;
  return(nb);
}

bool AIColsonPlayer::isStrongerPlayer(const Player* player)
{
  int i, nb;

  nb = getTotalArmiesOfPlayer(player);
  nb = nb + nb/5;
  PlayersArray::iterator it = m_game->playerList()->begin();
  PlayersArray::iterator it_end = m_game->playerList()->end();
  for (; it != it_end; it++)
  {
    if (getTotalArmiesOfPlayer(((Player*)(*it))) > nb)
    {
      return false;
    }
  }
  return true;
}

bool AIColsonPlayer::isSmallerPlayer(const Player* player)
{
  int nb = 3 * getTotalArmiesOfPlayer(player);
  PlayersArray::iterator it = m_game->playerList()->begin();
  PlayersArray::iterator it_end = m_game->playerList()->end();
  for (; it != it_end; it++)
  {
    if (getTotalArmiesOfPlayer(((Player*)(*it))) > nb)
    {
      return false;
    }
  }
  return false;
}

bool AIColsonPlayer::isContinentOfPlayer(int continent, const Player* player)
{
  return (m_world->getContinents().at(continent)->owner() == player);
}

void AIColsonPlayer::computeChoiceOfContinent(void)
{
  kdDebug() << "AIColsonPlayer::computeChoiceOfContinent" << endl;
  std::map< const Player*, std::map <const Continent*, int > > piCount;
  std::map< const Player*, std::map <int, bool > > maxContinent;
  std::map< const Player*, std::map <int, bool > > posContinent;
  int continent;
  int i, j, n, cont, min, max, bonus;

  PlayersArray::iterator it = m_game->playerList()->begin();
  PlayersArray::iterator it_end = m_game->playerList()->end();
  for (; it != it_end; it++)
  {
    m_piContinent[((Player*)(*it))] = -1;
    
    for (cont=0; cont<m_world->getContinents().count(); cont++)
    {
      piCount[((Player*)(*it))][m_world->getContinents().at(cont)] = 0;
      maxContinent[((Player*)(*it))][cont] = false;
      posContinent[((Player*)(*it))][cont] = false;
    }
  }

  for (i=0; i<m_world->getCountries().count(); i++)
  {
    piCount[m_world->getCountries().at(i)->owner()][m_world->getCountries().at(i)->continent()]++;
  }

  it = m_game->playerList()->begin();
  it_end = m_game->playerList()->end();
  for (; it != it_end; it++)
  {
    Player* i = ((Player*)(*it));
    min = 10000;
    max = 0;
    bonus = 0;
    for (cont=0; cont<m_world->getContinents().count(); cont++)
    {
      if (piCount[i][m_world->getContinents().at(cont)] > 0)
      {
        if (min == 0)
        {
          if ( (piCount[i][m_world->getContinents().at(cont)] == m_world->getContinents().at(cont)->getMembers().count())
                && (m_world->getContinents().at(cont)->getBonus()>bonus))
          {
            for (j=0; j<m_world->getContinents().count(); j++)
            {
              maxContinent[i][j] = false;
              posContinent[i][j] = false;
            }
            max = piCount[i][m_world->getContinents().at(cont)];
            bonus = m_world->getContinents().at(cont)->getBonus();
            maxContinent[i][cont] = true;
            m_piContinent[i] = cont;
          }
        }
        else if (piCount[i][m_world->getContinents().at(cont)] == m_world->getContinents().at(cont)->getMembers().count())
        {
          for (j=0; j<m_world->getContinents().count(); j++)
          {
            maxContinent[i][j] = false;
            posContinent[i][j] = false;
          }
          min = 0;
          max = piCount[i][m_world->getContinents().at(cont)];
          maxContinent[i][cont] = true;
          m_piContinent[i] = cont;
        }
        else if (piCount[i][m_world->getContinents().at(cont)] > max)
        {
          for (j=0; j<m_world->getContinents().count(); j++)
          {
            maxContinent[i][j] = false;
            if (piCount[i][m_world->getContinents().at(j)] < max-1)
                posContinent[i][j] = (piCount[i][m_world->getContinents().at(cont)] < m_world->getContinents().at(cont)->getMembers().count()/2);
            else
                posContinent[i][j] = true;
          }
          min = m_world->getContinents().at(cont)->getMembers().count() - piCount[i][m_world->getContinents().at(cont)];
          max = piCount[i][m_world->getContinents().at(cont)];
          bonus = m_world->getContinents().at(cont)->getBonus();
          maxContinent[i][cont] = true;
        }
        else if (   (piCount[i][m_world->getContinents().at(cont)] == max)
                  &&  isContinentOfMission(i, m_world->getContinents().at(cont)))
        {
          min = m_world->getContinents().at(cont)->getMembers().count() 
                - piCount[i][m_world->getContinents().at(cont)];
          bonus = 2 * m_world->getContinents().at(cont)->getBonus();
          maxContinent[i][cont] = true;
        }
        else if (   (piCount[i][m_world->getContinents().at(cont)] == max)
                  && (m_world->getContinents().at(cont)->getBonus() > bonus))
        {
          min = m_world->getContinents().at(cont)->getMembers().count() - piCount[i][m_world->getContinents().at(cont)];
          bonus = m_world->getContinents().at(cont)->getBonus();
          maxContinent[i][cont] = true;
        }
        else if (piCount[i][m_world->getContinents().at(cont)] >= m_world->getContinents().at(cont)->getMembers().count()/2)
        {
          posContinent[i][j] = true;
        }
      }
    }
  }

  it = m_game->playerList()->begin();
  it_end = m_game->playerList()->end();
  for (; it != it_end; it++)
  {
    KPlayer* n = *it;
    /* Search a continent with no conflict */
    PlayersArray::iterator it2 = m_game->playerList()->begin();
    PlayersArray::iterator it2_end = m_game->playerList()->end();
    for (; it2 != it2_end; it2++)
    {
      Player* j = static_cast<Player*>(*it2);
      continent = m_piContinent[j];
      if (continent == -1)
      {
        cont = 0;
        while ((cont<m_world->getContinents().count()) && (continent==-1))
        {
          if (maxContinent[j][cont])
          {
            continent = cont;
            i = 0;
            
            while (i<m_game->playerList()->count())
            {
              if (m_game->playerList()->at(i) != j) 
              {
                if ( m_piContinent[(Player*)(m_game->playerList()->at(i))] == cont)
                    continent = -1;
                else if (    (maxContinent[(Player*)(m_game->playerList()->at(i))][cont])
                          && (piCount[(Player*)(m_game->playerList()->at(i))][m_world->getContinents().at(cont)] > piCount[j][m_world->getContinents().at(cont)]))
                  continent = -1;
              }
              i++;
            }
          }
          cont++;
        }

        if (isFriendPlayer(j) && (continent!=-1))
        {
          for (cont=0; cont<m_world->getContinents().count(); cont++)
            if (cont != continent)
            {
              maxContinent[j][cont] = false;
              posContinent[j][cont] = false;
            }
          for (i=0; i!=m_game->playerList()->count(); i++)
              posContinent[(Player*)(m_game->playerList()->at(i))][continent] = false;
          m_piContinent[j] = continent;
        }
      }
    }

    /* Search a continent with no computer conflict */
    for (j=0; j<m_game->playerList()->count(); j++)
    {
      continent = m_piContinent[(Player*)(m_game->playerList()->at(j))];
      if (isFriendPlayer((Player*)(m_game->playerList()->at(j))) && (continent == -1))
      {
        cont = 0;
        while ((cont<m_world->getContinents().count()) && (continent==-1))
        {
          if (maxContinent[(Player*)(m_game->playerList()->at(j))][cont])
          {
            continent = cont;
            i = 0;
            while (i<m_game->playerList()->count())
            {
              if (isFriendPlayer((Player*)(m_game->playerList()->at(i))) && (i != j)) 
              {
                if ( m_piContinent[(Player*)(m_game->playerList()->at(i))] == cont)
                    continent = -1;
                else if (    (maxContinent[(Player*)(m_game->playerList()->at(i))][cont])
                          && (piCount[(Player*)(m_game->playerList()->at(i))][m_world->getContinents().at(cont)] > piCount[(Player*)(m_game->playerList()->at(j))][m_world->getContinents().at(cont)]))
                    continent = -1;
              }
              i++;
            }
          }
          cont++;
        }

        if (continent!=-1)
        {
          for (cont=0; cont!=m_world->getContinents().count(); cont++)
          //for (cont=0; cont!=m_world->getContinents().count(); cont++)
            if (cont != continent)
            {
              Player* pj = (Player*)(m_game->playerList()->at(j));
              maxContinent[pj][cont] = false;
              posContinent[pj][cont] = false;
            }
          for (i=0; i!=m_game->playerList()->count(); i++)
          {
              Player* pi = (Player*)(m_game->playerList()->at(i));
              posContinent[pi][continent] = false;
          }
          Player* pj = (Player*)(m_game->playerList()->at(j));
          m_piContinent[pj] = continent;
        }
      }
    }

    /* Search a possible continent with no conflict */
    for (j=0; j<m_game->playerList()->count(); j++)
    {
      Player* pj = (Player*)(m_game->playerList()->at(j));
      continent = m_piContinent[pj];
      if (isFriendPlayer(pj) && (continent == -1))
      {
        cont = 0;
        while ((cont<m_world->getContinents().count()) && (continent==-1))
        {
          if (posContinent[pj][cont])
          {
            continent = cont;
            i = 0;
            while (i<m_game->playerList()->count())
            {
              Player* pi = (Player*)(m_game->playerList()->at(i));
              if (isFriendPlayer(pi) && (i != j)) 
              {
                if ( m_piContinent[pi] == cont)
                    continent = -1;
                else if (    (maxContinent[pi][cont])
                          && (piCount[pi][m_world->getContinents().at(cont)] > piCount[pj][m_world->getContinents().at(cont)]))
                    continent = -1;
              }
              i++;
            }
          }
          cont++;
        }

        if (continent!=-1)
        {
          for (cont=0; cont!=m_world->getContinents().count(); cont++)
          //for (cont=0; cont!=m_world->getContinents().count(); cont++)
            if (cont != continent)
            {
              maxContinent[pj][cont] = false;
              posContinent[pj][cont] = false;
            }
          for (i=0; i!=m_game->playerList()->count(); i++)
          {
              Player* pi = (Player*)(m_game->playerList()->at(i));
              posContinent[pi][continent] = false;
          }
          m_piContinent[pj] = continent;
        }
      }
    }

    /* Search a possible continent with no computer conflict */
    for (j=0; j<m_game->playerList()->count(); j++)
    {
      Player* pj = (Player*)(m_game->playerList()->at(j));
      continent = m_piContinent[pj];
      if (isFriendPlayer(pj) && (continent == -1))
      {
        cont = 0;
        while ((cont<m_world->getContinents().count()) && (continent==-1))
        {
          if (posContinent[pj][cont])
          {
            continent = cont;
            i = 0;
            while (i<m_game->playerList()->count())
            {
              Player* pi = (Player*)(m_game->playerList()->at(i));
              if (isFriendPlayer(pi) && (i != j)) 
              {
                if ( m_piContinent[pi] == cont)
                    continent = -1;
                else if (    (maxContinent[pi][cont])
                          && (piCount[pi][m_world->getContinents().at(cont)] > piCount[pj][m_world->getContinents().at(cont)]))
                    continent = -1;
              }
              i++;
            }
          }
          cont++;
        }

        if (continent!=-1)
        {
          for (cont=0; cont!=m_world->getContinents().count(); cont++)
          //for (cont=0; cont!=m_world->getContinents().count(); cont++)
            if (cont != continent)
            {
              maxContinent[pj][cont] = false;
              posContinent[pj][cont] = false;
            }
          for (i=0; i!=m_game->playerList()->count(); i++)
          {
            Player* pi = (Player*)(m_game->playerList()->at(i));
            posContinent[pi][continent] = false;
          }
          m_piContinent[pj] = continent;
        }
      }
    }
  }

  for (j=0; j<m_game->playerList()->count(); j++)
  {
    Player* pj = (Player*)(m_game->playerList()->at(j));
    continent = m_piContinent[pj];

    if (isFriendPlayer(pj) && (continent == -1))
    {
      /* Search a continent */
      cont = 0;
      while ((cont<m_world->getContinents().count()) && (continent==-1))
      {
        if (piCount[pj][m_world->getContinents().at(cont)]>0)
        {
          continent = cont;
          i = 0;
          while (i<m_game->playerList()->count())
          {
            Player* pi = (Player*)(m_game->playerList()->at(i));
            if (i != j)
              if (isFriendPlayer(pi) && (i != j))
                  continent = -1;
            i++;
          }
        }
        cont++;
      }

      if (continent!=-1)
      {
        for (cont=0; cont!=m_world->getContinents().count(); cont++)
        //for (cont=0; cont!=m_world->getContinents().count(); cont++)
          if (cont!=continent)
          {
            maxContinent[pj][cont] = false;
            posContinent[pj][cont] = false;
          }
        for (i=0; i!=m_game->playerList()->count(); i++)
        {
            Player* pi = (Player*)(m_game->playerList()->at(i));
            posContinent[pi][continent] = false;
        }
        m_piContinent[pj] = continent;
      }
    }
  }

  for (j=0; j<m_game->playerList()->count(); j++)
  {
    Player* pj = (Player*)(m_game->playerList()->at(j));
    continent = m_piContinent[pj];

    /* Search a continent */
    cont = 0;
    while ((cont<m_world->getContinents().count()) && (continent==-1))
    {
      if (maxContinent[pj][cont])
          continent = cont;
      cont++;
    }

    if (continent!=-1)
    {
      for (cont=0; cont!=m_world->getContinents().count(); cont++)
      //for (cont=0; cont!=m_world->getContinents().count(); cont++)
        if (cont!=continent)
        {
          maxContinent[pj][cont] = false;
          posContinent[pj][cont] = false;
        }
      for (i=0; i!=m_game->playerList()->count(); i++)
      {
          Player* pi = (Player*)(m_game->playerList()->at(i));
          posContinent[pi][continent] = false;
      }
      m_piContinent[pj] = continent;
    }
  }
  kdDebug() << "AIColsonPlayer::computeChoiceOfContinent DONE" << endl;
}

/**
  * Gets the continent already selected for this player
  * @param attack This is the number of countries of the selected continent
  * that have at least one adjacent enemy
  */
int AIColsonPlayer::GetContinentToFortify(int *attack)
{
  kdDebug() << "AIColsonPlayer::GetContinentToFortify" << endl;
  int continent = -1;

  continent = m_piContinent[this];
  *attack = 0;
  for (int i=0; i!=m_world->getCountries().count(); i++)
  {
    if (    (RISK_GetOwnerOfCountry(i) == this)
           && (RISK_GetContinentOfCountry(i) == continent)
           &&  GAME_IsEnemyAdjacent(i))
    {
      (*attack)++;
    }
  }
  kdDebug() << "AIColsonPlayer::GetContinentToFortify found " << continent << endl;
  return(continent);
}

/** 
  * @param attack Number of countries with adjacent ennemies in the selected
  * continent
  */
int AIColsonPlayer::GetContinentToConquier(int *attack)
{
  int  piCount[m_world->getContinents().count()];
  int  piOppo[m_world->getContinents().count()];
  int  piAttac[m_world->getContinents().count()];
  int  i, min, max, bonus, continent;

  /* Init. */
  for (int cont=0; cont!=m_world->getContinents().count(); cont++)
  //for (i=0; i!=m_world->getContinents().count(); i++)
    {
       piCount[cont] = 0;
       piOppo[cont] = 0;
       piAttac[cont] = 0;
    }

  /* Count up how many countries the player has in each of the continents */
  for (i=0; i!=m_world->getCountries().count(); i++)
    {
      if (RISK_GetOwnerOfCountry(i) == this)
        {
	  piCount[RISK_GetContinentOfCountry(i)]++;
          piOppo[RISK_GetContinentOfCountry(i)] -= RISK_GetNumArmiesOfCountry (i);
          if (GAME_IsEnemyAdjacent(i))
	      piAttac[RISK_GetContinentOfCountry(i)]++;
	}
      else
        {
          if (isEnemyPlayer(RISK_GetOwnerOfCountry(i)))
              piOppo[RISK_GetContinentOfCountry(i)] += RISK_GetNumArmiesOfCountry (i);
          else
              piOppo[RISK_GetContinentOfCountry(i)] += 2 * RISK_GetNumArmiesOfCountry (i);
        }
    }

  continent = -1;
  min = 10000;
  max = 0;
  bonus = 0;
  for (i=0; i!=m_world->getContinents().count(); i++)
  //for (i=0; i!=m_world->getContinents().count(); i++)
    {
      if (isContinentOfMission(this, m_world->getContinents().at(i)))
          piOppo[i] -= piOppo[i]/3;
      if (piCount[i] < m_world->getContinents().at(i)->getMembers().count())
          if (piOppo[i] < min)
            {
               continent = i;
               min = piOppo[i];
               max = piCount[i];
               bonus = m_world->getContinents().at(i)->getBonus();
            }
    }

  *attack = piAttac[continent];
  return(continent);
}

int AIColsonPlayer::NbEnenyAdjacent(int iCountry)
{
  int i, max, nb, destCountry;
  std::map<Player*, int> NumEnemy;
  std::map<Player*, bool> fIsEnemy;
  bool  fIAmStrong;

  Player* player = RISK_GetOwnerOfCountry(iCountry);
  fIAmStrong = isStrongerPlayer(player);
  for (i=0; i<m_game->playerList()->count(); i++)
  {
    fIsEnemy[(Player*)m_game->playerList()->at(i)] = false;
  }
  i = 0;
  Player* iEnemy = 0; 
  while ((i < 6) && (RISK_GetAdjCountryOfCountry(iCountry, i) != -1))
    {
      destCountry = RISK_GetAdjCountryOfCountry(iCountry, i);
      iEnemy = RISK_GetOwnerOfCountry(destCountry);
      if (iEnemy != player)
        {
          if (NumEnemy[iEnemy] < RISK_GetNumArmiesOfCountry(destCountry))
            NumEnemy[iEnemy] = RISK_GetNumArmiesOfCountry(destCountry);
          fIsEnemy[iEnemy] = true;
        }
      i++;
    }
  max = 0;
  for (int ci=0; i<m_game->playerList()->count(); ci++)
    {
      Player* i = (Player*)m_game->playerList()->at(ci);
      if ((i != player) && fIsEnemy[i])
        {
          nb = NumEnemy[iEnemy];
          if (!fIAmStrong && isStrongerPlayer(i))
              nb = nb + 10;
          if (isEnemyPlayer(i))
              nb = nb + 5;
          if (nb > max)
              max = nb;
        }
    }
  return max;
}

int AIColsonPlayer::NbToEqualEnenyAdjacent(int iCountry)
{
  int nbe = 0;
  int nb = 0;
  int i = 0;
  while ((i < 6) && (RISK_GetAdjCountryOfCountry(iCountry, i) != -1))
  {
    int destCountry = RISK_GetAdjCountryOfCountry(iCountry, i);
    Player* iEnemy = RISK_GetOwnerOfCountry(destCountry);
    if (iEnemy != this)
      {
        nbe -= RISK_GetNumArmiesOfCountry(destCountry);
        nb++;
      }
    i++;
  }
  int nbi = RISK_GetNumArmiesOfCountry(iCountry);
  nbe = (nbe + nbi)/nb;
//   if ((nbi - nbe)< 10)
//       nbe = nbi - 10;
  kdDebug() << "NbToEqualEnenyAdjacent of " << iCountry << " is " << nbe << endl;
  return nbe;
}


/************************************************************************/

bool AIColsonPlayer::ComputerAttack(int destCountry, bool die, int dif, int iMove)
{
  kdDebug() << "AIColsonPlayer::ComputerAttack " << destCountry << endl;

  if (Attack_SrcCountry!=-1 && Attack_DestCountry!=-1
      && (RISK_GetOwnerOfCountry(Attack_DestCountry) != this)
          && (RISK_GetNumArmiesOfCountry(Attack_SrcCountry) > 1)
          && (die || (   RISK_GetNumArmiesOfCountry(Attack_SrcCountry)
                       > RISK_GetNumArmiesOfCountry(Attack_DestCountry))))
  {
    kdDebug() << "    Retry attack" << endl;
    return true;
  }

  int srcCountry = -1;
  int max = RISK_GetNumArmiesOfCountry(destCountry) + dif;
  int i = 0;
  while ((i < 6) && (RISK_GetAdjCountryOfCountry(destCountry, i) != -1))
  {
    int iCountry = RISK_GetAdjCountryOfCountry(destCountry, i);
    if ( (RISK_GetOwnerOfCountry(iCountry) == this)
        && (RISK_GetNumArmiesOfCountry(iCountry) > max) )
    {
      max = RISK_GetNumArmiesOfCountry(iCountry);
      srcCountry = iCountry;
    }
    i++;
  }
  if (srcCountry == -1)
  {
    Attack_SrcCountry = -1;
    Attack_DestCountry = -1;
    return false;
  }
  kdDebug() << "    srcCountry = " << srcCountry << endl;

  Attack_SrcCountry = srcCountry;
  Attack_DestCountry = destCountry;
  AI_Attack (srcCountry, destCountry, ATTACK_DOORDIE, DICE_MAXIMUM, iMove);
  return true;
}

bool AIColsonPlayer::Fortify()
{
  kdDebug() << "AIColsonPlayer::Fortify" << endl;
  int continent;
  int iCountry, nb;
  bool  boool;

  continent = GetContinentToFortify(&nb);
  boool = false;
  iCountry = 0;
  if (nb > 0)
  {
    nb = rand() % nb;
    while (!boool && (iCountry < m_world->getCountries().count()))
    {
      if ((RISK_GetOwnerOfCountry(iCountry) == this) &&
          (RISK_GetContinentOfCountry(iCountry) == continent) &&
          GAME_IsEnemyAdjacent(iCountry))
      {
        if (nb <= 0)
        {
          AI_Place (iCountry, 1);
          return true;
          boool = true;
        }
        nb--;
      }
      if (!boool)
        iCountry++;
    }
  }
  iCountry = 0;
  while (!boool && (iCountry < m_world->getCountries().count()))
  {
    if (    (RISK_GetOwnerOfCountry(iCountry) == this)
          &&  GAME_IsEnemyAdjacent(iCountry))
    {
      AI_Place (iCountry, 1);
      return true;
      boool = true;
    }
    else
      iCountry++;
  }
  iCountry = 0;
  while (!boool && (iCountry < m_world->getCountries().count()))
  {
    if (RISK_GetOwnerOfCountry(iCountry) == this)
    {
      AI_Place (iCountry, 1);
      return true;
      boool = true;
    }
    else
      iCountry++;
  }
  return false;
}

bool AIColsonPlayer::Place()
{
  kdDebug() << "AIColsonPlayer::Place" << endl;

  /* Try to destroy a weak enemy player. 
   * NOTE this could be an error
   * if destroying this player is another player goal... */
  kdDebug() << "AIColsonPlayer::Place 1. Try to destroy a enemy player" << endl;
  
  for (int iCountry = 0; iCountry < m_world->getCountries().count(); iCountry++)
  {
    if ((RISK_GetOwnerOfCountry(iCountry) == this) &&
          GAME_IsEnemyAdjacent(iCountry))
    {
      int i = 0;
      while ((i < 6) && (RISK_GetAdjCountryOfCountry(iCountry, i) != -1))
      {
        int destCountry = RISK_GetAdjCountryOfCountry(iCountry, i);
        Player* iEnemy = RISK_GetOwnerOfCountry(destCountry);
        if (    (iEnemy != this) && isEnemyPlayer(iEnemy)
              && (getTotalArmiesOfPlayer(iEnemy)
                  < (   RISK_GetNumArmiesOfPlayer(this)
                      + RISK_GetNumArmiesOfCountry(iCountry) - 5)))
        {
          AI_Place (iCountry, 1);
//           AI_Place (iCountry, RISK_GetNumArmiesOfPlayer(this));
          return true;
        }
        i++;
      }
    }
  }

  int nbCountriesWithAdjacentEnemies;
  int continent = GetContinentToFortify(&nbCountriesWithAdjacentEnemies);
  /* Try to conquier an entire continent, attack enemy */
  kdDebug() << "AIColsonPlayer::Place 2. Try to conquier an entire continent, attack enemy" << endl;
  
  for (int destCountry = 0; 
       destCountry < m_world->getCountries().count();
       destCountry++)
  {
    Player* iEnemy = RISK_GetOwnerOfCountry(destCountry);
    if (    (iEnemy != this) && isEnemyPlayer(iEnemy)
          && (RISK_GetContinentOfCountry(destCountry) == continent))
    {
      int i = 0;
      while ((i < 6) && (RISK_GetAdjCountryOfCountry(destCountry, i) != -1))
      {
        int iCountry = RISK_GetAdjCountryOfCountry(destCountry, i);
        if ( (RISK_GetOwnerOfCountry(iCountry) == this)
            && (RISK_GetNumArmiesOfCountry(iCountry) < (1.5*RISK_GetNumArmiesOfCountry(destCountry))+3)
            )
        {
          AI_Place (iCountry, 1);
          return true;
        }
        i++;
      }
    }
  }

  /* Try to destroy a player */
  kdDebug() << "AIColsonPlayer::Place 3. Try to destroy a player" << endl;
  for (int iCountry = 0;
       iCountry < m_world->getCountries().count();
       iCountry++)
  {
    if ((RISK_GetOwnerOfCountry(iCountry) == this) &&
          GAME_IsEnemyAdjacent(iCountry))
    {
      int i = 0;
      while ((i < 6) && (RISK_GetAdjCountryOfCountry(iCountry, i) != -1))
      {
        int destCountry = RISK_GetAdjCountryOfCountry(iCountry, i);
        Player* iEnemy = RISK_GetOwnerOfCountry(destCountry);
        if (    (iEnemy != this)
              && (getTotalArmiesOfPlayer(iEnemy)
                  < (   RISK_GetNumArmiesOfPlayer(this)
                      + RISK_GetNumArmiesOfCountry(iCountry) - 5))
              && (RISK_GetNumArmiesOfCountry(iCountry) < (1.5*RISK_GetNumArmiesOfCountry(destCountry))+3)

            )
        {
          AI_Place (iCountry, 1);
          return true;
        }
        i++;
      }
    }
  }

  /* Try to conquier an entire continent */
  kdDebug() << "AIColsonPlayer::Place 4. Try to conquier an entire continent" << endl;
  for (int destCountry = 0; 
       destCountry < m_world->getCountries().count();
       destCountry++)
  {
    Player* iEnemy = RISK_GetOwnerOfCountry(destCountry);
    if (    (RISK_GetContinentOfCountry(destCountry) == continent)
          && (iEnemy != this))
    {
      int i = 0;
      while ((i < 6) && (RISK_GetAdjCountryOfCountry(destCountry, i) != -1))
      {
        int iCountry = RISK_GetAdjCountryOfCountry(destCountry, i);
        if (RISK_GetOwnerOfCountry(iCountry) == this
            && (RISK_GetNumArmiesOfCountry(iCountry) < (1.5*RISK_GetNumArmiesOfCountry(destCountry))+3)
            )
        {
          AI_Place (iCountry, 1);
          return true;
        }
        i++;
      }
    }
  }

  /* Try to defend an entire continent */
  kdDebug() << "AIColsonPlayer::Place 5. Try to defend an entire continent" << endl;
  if ( (nbCountriesWithAdjacentEnemies > 0) 
        && (RISK_GetNumArmiesOfPlayer(this) > 0) )
  {
    int min = 0;
    bool boool = false;
    int destCountry;
    for (int iCountry = 0; 
         iCountry < m_world->getCountries().count();
         iCountry++)
    {
      if (    (RISK_GetOwnerOfCountry(iCountry) == this)
            && (RISK_GetContinentOfCountry(iCountry) == continent)
            &&  GAME_IsEnemyAdjacent(iCountry))
      {
        int nb = NbToEqualEnenyAdjacent(iCountry);
        if (nb < min)
        {
          min = nb ;
          destCountry = iCountry;
          boool = true;
        }
      }
    }
    if (boool)
    {
      AI_Place (destCountry, 1);
      return true;
    }
  }

  kdDebug() << "AIColsonPlayer::Place 6. Try to conquier an entire continent, attack enemy" << endl;
  continent = GetContinentToConquier(&nbCountriesWithAdjacentEnemies);
  /* Try to conquier an entire continent, attack enemy */
  for (int iCountry = 0;
       iCountry < m_world->getCountries().count();
       iCountry++)
  {
    if ((RISK_GetOwnerOfCountry(iCountry) == this) &&
        (RISK_GetContinentOfCountry(iCountry) == continent) &&
        GAME_IsEnemyAdjacent(iCountry))
    {
      int i = 0;
      while ((i < 6) && (RISK_GetAdjCountryOfCountry(iCountry, i) != -1))
      {
        int destCountry = RISK_GetAdjCountryOfCountry(iCountry, i);
        Player* iEnemy = RISK_GetOwnerOfCountry(destCountry);
        if (    (iEnemy != this) && isEnemyPlayer(iEnemy)
              && (RISK_GetContinentOfCountry(destCountry) == continent)
              && (RISK_GetNumArmiesOfCountry(iCountry) < (1.5*RISK_GetNumArmiesOfCountry(destCountry))+3)
            )
        {
          AI_Place (iCountry, 1);
          return true;
        }
        i++;
      }
    }
  }

  kdDebug() << "AIColsonPlayer::Place 7. Try to conquier an entire continent" << endl;
  /* Try to conquier an entire continent */
  for (int   iCountry = 0;
       iCountry < m_world->getCountries().count();
       iCountry++)
  {
    if ((RISK_GetOwnerOfCountry(iCountry) == this) &&
        (RISK_GetContinentOfCountry(iCountry) == continent) &&
          GAME_IsEnemyAdjacent(iCountry))
    {
      int i = 0;
      while ((i < 6) && (RISK_GetAdjCountryOfCountry(iCountry, i) != -1))
      {
        int destCountry = RISK_GetAdjCountryOfCountry(iCountry, i);
        Player* iEnemy = RISK_GetOwnerOfCountry(destCountry);
        if (    (iEnemy != this)
              && (RISK_GetContinentOfCountry(destCountry) == continent)
              && (RISK_GetNumArmiesOfCountry(iCountry) < (1.5*RISK_GetNumArmiesOfCountry(destCountry))+3)
            )
        {
          AI_Place (iCountry, 1);
          return true;
        }
        i++;
      }
    }
  }

  kdDebug() << "AIColsonPlayer::Place 8. Try to defend an entire continent" << endl;
  /* Try to defend an entire continent */
  if (nbCountriesWithAdjacentEnemies > 0)
  {
    int min = 2; // a la place de 1000
    bool boool = false;
    int destCountry;
    for (int iCountry = 0;
         iCountry < m_world->getCountries().count();
         iCountry++)
    {
      if (    (RISK_GetOwnerOfCountry(iCountry) == this)
          && (RISK_GetContinentOfCountry(iCountry) == continent)
          &&  GAME_IsEnemyAdjacent(iCountry))
      {
        int nb = NbToEqualEnenyAdjacent(iCountry);
        if (nb < min)
        {
//           min = RISK_GetNumArmiesOfCountry(iCountry);
          min = nb;
          destCountry = iCountry;
          boool = true;
        }
      }
    }
    if (boool)
    {
      AI_Place (destCountry, 1);
      return true;
    }
  }

  kdDebug() << "AIColsonPlayer::Place 9. Try to prepare an enemy attack, find a lowest defence" << endl;
  /* Try to prepare an enemy attack, find a lowest defence */
  int selected = 0;
  int myDefenceDelta = std::numeric_limits<int>::max();
  bool boool9 = false;
  for (int iCountry = 0;
       iCountry < m_world->getCountries().count(); 
       iCountry++)
  {
    if (    (RISK_GetOwnerOfCountry(iCountry) == this)
        &&  GAME_IsEnemyAdjacent(iCountry))
    {
      int i = 0;
      while ((i < 6) && (RISK_GetAdjCountryOfCountry(iCountry, i) != -1))
      {
        int destCountry = RISK_GetAdjCountryOfCountry(iCountry, i);
        int delta = RISK_GetNumArmiesOfCountry(iCountry) - RISK_GetNumArmiesOfCountry(destCountry);
        Player* iEnemy = RISK_GetOwnerOfCountry(destCountry);
        if (    (RISK_GetContinentOfCountry(destCountry) == continent)
             && (iEnemy != this) && isEnemyPlayer(iEnemy)
             && ( (   RISK_GetNumArmiesOfCountry(iCountry) < 
                      (1.5*RISK_GetNumArmiesOfCountry(destCountry))+3 ) 
                   || (RISK_GetNumArmiesOfCountry(iCountry)==1) ) 
              && (delta < myDefenceDelta ) )
        {
          selected = iCountry;
          myDefenceDelta = delta;
          boool9 = true;
        }
        i++;
      }
    }
  }
  if (boool9)
  {
    AI_Place (selected, 1);
    return true;
  }

  kdDebug() << "AIColsonPlayer::Place 10.Try to prepare an enemy attack" << endl;
  /* Try to prepare an enemy attack */
  bool boool10 = false;
  myDefenceDelta = std::numeric_limits<int>::max();
  for (int iCountry = 0; 
       iCountry < m_world->getCountries().count();
       iCountry++)
  {
    kdDebug() << "    looking at iCountry " << iCountry << endl;
    kdDebug() << "    owner " << RISK_GetOwnerOfCountry(iCountry) << endl;
    if (    (RISK_GetOwnerOfCountry(iCountry) == this)
          &&  GAME_IsEnemyAdjacent(iCountry))
    {
      int i = 0;
      while ((i < 6) && (RISK_GetAdjCountryOfCountry(iCountry, i) != -1))
      {
        int destCountry = RISK_GetAdjCountryOfCountry(iCountry, i);
        int delta = RISK_GetNumArmiesOfCountry(iCountry) - RISK_GetNumArmiesOfCountry(destCountry);
        Player* iEnemy = RISK_GetOwnerOfCountry(destCountry);
        kdDebug() << "        looking at destCountry " << destCountry << endl;
        kdDebug() << "        iEnemy " << RISK_GetOwnerOfCountry(destCountry) << endl;
        kdDebug() << "        isEnemyPlayer? " << isEnemyPlayer(iEnemy) << endl;
        kdDebug() << "    delta / myDefenceDelta " << delta << " / " << myDefenceDelta << endl;
        if ((iEnemy != this) && isEnemyPlayer(iEnemy) 
              && (delta < myDefenceDelta ) )
        {
          kdDebug() << "    GOT IT " << iCountry << endl;
          myDefenceDelta = delta;
          boool10 = true;
          selected = iCountry;
        }
        i++;
      }
    }
  }
  if (boool10)
  {
    AI_Place (selected, 1);
    //AI_Place (iCountry, RISK_GetNumArmiesOfPlayer(this));
    return true;
  }

  // cannot reach this line ???
  assert(false);

  kdDebug() << "AIColsonPlayer::Place 11.Try to prepare an attack" << endl;
  /* Try to prepare an attack */
  for (int iCountry = 0;
       iCountry < m_world->getCountries().count();
       iCountry++)
  {
    if (    (RISK_GetOwnerOfCountry(iCountry) == this)
          &&  GAME_IsEnemyAdjacent(iCountry))
    {
      AI_Place (iCountry, 1);
      return true;
    }
  }

  kdDebug() << "AIColsonPlayer::Place 12.Try to place" << endl;
  /* Try to place */
  for (int iCountry = 0; 
       iCountry < m_world->getCountries().count(); 
       iCountry++)
  {
    if (RISK_GetOwnerOfCountry(iCountry) == this)
    {
      AI_Place (iCountry, 1);
      return true;
    }
  }
  return false;
}

/** @return KsirK change: true if an attack have been tempted; false otherwise */
bool AIColsonPlayer::AttackEnemy()
{
  kdDebug() << "AIColsonPlayer::AttackEnemy" << endl;
  int continent, destCountry, nbCountriesWithAdjacentEnemies;

  if (Attack_SrcCountry!=-1 && Attack_DestCountry != -1
      && (RISK_GetOwnerOfCountry(Attack_DestCountry)!=this)
      && ComputerAttack (Attack_DestCountry, true,
                            (RISK_GetNumArmiesOfCountry(Attack_DestCountry) < 5)?1:
                                ((nbCountriesWithAdjacentEnemies > 4)?RISK_GetNumArmiesOfCountry(Attack_DestCountry):3),
                            ARMIES_MOVE_MANUAL))
  {
    kdDebug() << "Attack tempted again." << endl;
    return true;
  }


  continent = GetContinentToFortify(&nbCountriesWithAdjacentEnemies);

  kdDebug() << "AIColsonPlayer::AttackEnemy Try to conquier an entire continent, attack player of other species " << endl;
  /* Try to conquier an entire continent, attack player of other species */
  destCountry = 0;
  while (destCountry < m_world->getCountries().count())
  {
//     kdDebug() << "destCountry=" << destCountry << " / m_world->getCountries().count()=" << m_world->getCountries().count() << endl;
    Player* iEnemy = RISK_GetOwnerOfCountry(destCountry);
    if (    (RISK_GetContinentOfCountry(destCountry) == continent)
          && (iEnemy != this) && isEnemyPlayer(iEnemy))
    {
      if (ComputerAttack (destCountry, true,
                                (RISK_GetNumArmiesOfCountry(destCountry) < 5)?1:
                                    ((nbCountriesWithAdjacentEnemies > 4)?RISK_GetNumArmiesOfCountry(destCountry):3),
                                ARMIES_MOVE_MANUAL))
      {
        destCountry = 0;
        kdDebug() << "Attack tempted." << endl;
        return true;
      }
      else
      {
        destCountry++;
      }
    }
    else
    {
      destCountry++;
    }
  }

  kdDebug() << "AIColsonPlayer::AttackEnemy Try to conquier an entire continent " << endl;
  /* Try to conquier an entire continent */
  destCountry = 0;
  while (destCountry < m_world->getCountries().count())
  {
    if (    (RISK_GetContinentOfCountry(destCountry) == continent)
          && (RISK_GetOwnerOfCountry(destCountry) != this))
    {
      if (ComputerAttack (destCountry, true,
                          (RISK_GetNumArmiesOfCountry(destCountry) < 3)?1:50,
                            ARMIES_MOVE_MANUAL))
      {
        destCountry = 0;
        kdDebug() << "Attack tempted." << endl;
        return true;
      }
      else
        destCountry++;
    }
    else
      destCountry++;
  }

  if (    !isContinentOfPlayer(continent, this)
       && (m_numTurn[this] <= 2)
       && (allPlayers.count() > m_world->getContinents().count()/2))
  {
    kdDebug() << "AIColsonPlayer::AttackEnemy No attack tried" << endl;
    Attack_SrcCountry = -1;
    Attack_DestCountry = -1;
    return false;
  }

  continent = GetContinentToConquier(&nbCountriesWithAdjacentEnemies);

  kdDebug() << "AIColsonPlayer::AttackEnemy Try to destroy a human player" << endl;
  /* Try to destroy a human player */
  destCountry = 0;
  while (destCountry < m_world->getCountries().count())
  {
    if (    (RISK_GetOwnerOfCountry(destCountry) != this)
          && (!RISK_GetOwnerOfCountry(destCountry)->isAI())
          && (RISK_GetNumCountriesOfPlayer(RISK_GetOwnerOfCountry(destCountry)) == 1))
    {
        if (ComputerAttack (destCountry, true,
                                  (nbCountriesWithAdjacentEnemies > 2)?10:2, ARMIES_MOVE_MANUAL))
        {
          kdDebug() << "Attack tempted." << endl;
          return true;
        }
    }
    destCountry++;
  }

  kdDebug() << "AIColsonPlayer::AttackEnemy Try to destroy a enemy player" << endl;
  /* Try to destroy a enemy player */
  destCountry = 0;
  while (destCountry < m_world->getCountries().count())
  {
    if (   isEnemyPlayer(RISK_GetOwnerOfCountry(destCountry))
          && (RISK_GetNumCountriesOfPlayer(RISK_GetOwnerOfCountry(destCountry)) == 1))
    {
      if (ComputerAttack (destCountry, true,
                                (nbCountriesWithAdjacentEnemies > 2)?20:2, ARMIES_MOVE_MANUAL))
      {
        kdDebug() << "Attack tempted." << endl;
        return true;
      }
    }
    destCountry++;
  }

  kdDebug() << "AIColsonPlayer::AttackEnemy Try to destroy a player" << endl;
  /* Try to destroy a player */
  destCountry = 0;
  while (destCountry < m_world->getCountries().count())
  {
    if (    (RISK_GetOwnerOfCountry(destCountry) != this)
          && (RISK_GetNumCountriesOfPlayer(RISK_GetOwnerOfCountry(destCountry)) == 1))
    {
      if (ComputerAttack (destCountry, true,
                                (nbCountriesWithAdjacentEnemies> 2)?20:2, ARMIES_MOVE_MANUAL))
      {
        kdDebug() << "Attack tempted." << endl;
        return true;
      }
    }
    destCountry++;
  }

  kdDebug() << "AIColsonPlayer::AttackEnemy Try to conquier an entire continent, attack player of other species" << endl;
  /* Try to conquier an entire continent, attack player of other species */
  destCountry = 0;
  while (destCountry < m_world->getCountries().count())
  {
    if (    (RISK_GetContinentOfCountry(destCountry) == continent)
          && (RISK_GetOwnerOfCountry(destCountry) != this)
          &&  isEnemyPlayer(RISK_GetOwnerOfCountry(destCountry)))
      {
        if (ComputerAttack (destCountry, true,
                                  (RISK_GetNumArmiesOfCountry(destCountry) < 5)?1:
                                      ((nbCountriesWithAdjacentEnemies > 4)?RISK_GetNumArmiesOfCountry(destCountry):3),
                                  ARMIES_MOVE_MANUAL))
          {
            destCountry = 0;
            kdDebug() << "Attack tempted." << endl;
            return true;
          }
        else
            destCountry++;
      }
    else
        destCountry++;
  }

  kdDebug() << "AIColsonPlayer::AttackEnemy Try to conquier an entire continent" << endl;
  /* Try to conquier an entire continent */
  destCountry = 0;
  while (destCountry < m_world->getCountries().count())
  {
    if (    (RISK_GetContinentOfCountry(destCountry) == continent)
          && (RISK_GetOwnerOfCountry(destCountry) != this))
      {
        if (ComputerAttack (destCountry, true,
                            (RISK_GetNumArmiesOfCountry(destCountry) < 3)?1:50,
                              ARMIES_MOVE_MANUAL))
          {
            destCountry = 0;
            kdDebug() << "Attack tempted." << endl;
            return true;
          }
        else
            destCountry++;
      }
    else
        destCountry++;
  }

  kdDebug() << "AIColsonPlayer::AttackEnemy Try to attack a stronger human player for a card" << endl;
  /* Try to attack a stronger human player for a card */
  destCountry = 0;
  while (destCountry < m_world->getCountries().count())
  {
    Player* iEnemy = RISK_GetOwnerOfCountry(destCountry);
    if (    (iEnemy != this) && isEnemyPlayer(iEnemy)
          && (!iEnemy->isAI())
          && isStrongerPlayer(RISK_GetOwnerOfCountry(destCountry)))
    {
      if (ComputerAttack (destCountry, false,
                          (RISK_GetNumArmiesOfCountry(destCountry) < 3)?1:((nbCountriesWithAdjacentEnemies > 2)?10:2),
                          ARMIES_MOVE_MANUAL))
      {
        kdDebug() << "Attack tempted." << endl;
        return true;
      }
    }
    destCountry++;
  }

  kdDebug() << "AIColsonPlayer::AttackEnemy Try to attack an human player for a card" << endl;
  /* Try to attack an human player for a card */
  destCountry = 0;
  while (destCountry < m_world->getCountries().count())
  {
    Player* iEnemy = RISK_GetOwnerOfCountry(destCountry);
    if (    (iEnemy != this) && isEnemyPlayer(iEnemy)
          && (!iEnemy->isAI()))
    {
      if (ComputerAttack (destCountry, false,
                          (RISK_GetNumArmiesOfCountry(destCountry) < 3)?1:((nbCountriesWithAdjacentEnemies > 2)?10:2),
                          ARMIES_MOVE_MANUAL))
      {
        kdDebug() << "Attack tempted." << endl;
        return true;
      }
    }
    destCountry++;
  }

  kdDebug() << "AIColsonPlayer::AttackEnemy Try to attack enemy player for a card" << endl;
  /* Try to attack enemy player for a card */
  destCountry = 0;
  while (destCountry < m_world->getCountries().count())
  {
    Player* iEnemy = RISK_GetOwnerOfCountry(destCountry);
    if (    (iEnemy != this) && isEnemyPlayer(iEnemy))
        if (ComputerAttack (destCountry, false,
                            (RISK_GetNumArmiesOfCountry(destCountry) < 3)?1:((nbCountriesWithAdjacentEnemies > 2)?10:2),
                            ARMIES_MOVE_MANUAL))
    {
      {
        kdDebug() << "Attack tempted." << endl;
        return true;
      }
    }
    destCountry++;
  }

  kdDebug() << "AIColsonPlayer::AttackEnemy Try to attack for a card, attack a stronger player" << endl;
  /* Try to attack for a card, attack a stronger player */
  destCountry = 0;
  while ((destCountry < m_world->getCountries().count()))
  {
    Player* iEnemy = RISK_GetOwnerOfCountry(destCountry);
    if (    (iEnemy != this)
          && isStrongerPlayer(iEnemy))
    {
      if (ComputerAttack (destCountry, false,
                          (RISK_GetNumArmiesOfCountry(destCountry) < 2)?1:100,
                          ARMIES_MOVE_MANUAL))
      {
        kdDebug() << "Attack tempted." << endl;
        return true;
      }
    }
    destCountry++;
  }

  kdDebug() << "AIColsonPlayer::AttackEnemy Try to attack for a card" << endl;
  /* Try to attack for a card */
  destCountry = 0;
  while ((destCountry < m_world->getCountries().count()))
  {
    if (RISK_GetOwnerOfCountry(destCountry) != this)
    {
      if (ComputerAttack (destCountry, false,
                          (RISK_GetNumArmiesOfCountry(destCountry) < 2)?1:100,
                          ARMIES_MOVE_MIN))
      {
        kdDebug() << "Attack tempted." << endl;
        return true;
      }
    }
    destCountry++;
  }

  kdDebug() << "AIColsonPlayer::AttackEnemy No attack tried" << endl;
  // No attack tried
  Attack_SrcCountry = -1;
  Attack_DestCountry = -1;
  return false;
}

bool AIColsonPlayer::Attack()
{
  kdDebug() << "AIColsonPlayer::Attack" << endl;
  int enemyAlive;

//   if (RISK_GetAttackModeOfPlayer (player) != ACTION_DOORDIE)
//       RISK_SetAttackModeOfPlayer (player, ACTION_DOORDIE);
//   if (RISK_GetDiceModeOfPlayer (player) != ATTACK_AUTO)
//       RISK_SetDiceModeOfPlayer (player, ATTACK_AUTO);

  enemyAlive = getNumEnemy();
  while ((enemyAlive == 0) && (m_levelEnemy >0))
    {
      m_levelEnemy--;
      enemyAlive = getNumEnemy();
    }

  return AttackEnemy();
}

void AIColsonPlayer::HowManyArmiesToMove(int *nb)
{
  if ((Attack_SrcCountry == -1) || (Attack_DestCountry == -1))
      return;

  if (!GAME_IsEnemyAdjacent(Attack_SrcCountry))
      *nb = 0;
  else if (!GAME_IsEnemyAdjacent(Attack_DestCountry))
      *nb = *nb;
  else
      *nb = *nb/2;
  Attack_SrcCountry  = -1;
  Attack_DestCountry = -1;
}

int AIColsonPlayer::FindEnemyAdjacent(int iCountry, int distance)
{
  int i, min, res, dest;

  Player* iPlayer = RISK_GetOwnerOfCountry(iCountry);
  min = 100000;
  for (i=0; i!=6 && RISK_GetAdjCountryOfCountry(iCountry, i)!=-1; i++)
    {
      dest = RISK_GetAdjCountryOfCountry(iCountry, i);
      if (RISK_GetOwnerOfCountry(dest) == iPlayer)
        {
          if (distance <= 3)
            {
              res = FindEnemyAdjacent(dest, distance + 1);
              if (res < min)
                  min = res;
            }
        }
      else
          min = 0;
    }

  return (min + 1);
}

int AIColsonPlayer::GAME_FindEnemyAdjacent(int iCountry)
{
  int i, min, res, dest, destCountry;
  Player* iPlayer = RISK_GetOwnerOfCountry(iCountry);
  destCountry = -1;
  min = 100000;
  for (i=0; i!=6 && RISK_GetAdjCountryOfCountry(iCountry, i)!=-1; i++)
    {
      dest = RISK_GetAdjCountryOfCountry(iCountry, i);
      if (RISK_GetOwnerOfCountry(dest) == iPlayer)
        {
          res = FindEnemyAdjacent(dest, 0);
          if (res < min)
            {
              min = res;
              destCountry = dest;
            }
        }
      else
          min = 0;
    }

  return (destCountry);
}


bool AIColsonPlayer::Move()
{
  kdDebug() << "AIColsonPlayer::Move" << endl;
  int iCountry, destCountry, i, max;
  bool  boool;

  /* Try to move an unused max army in a frontier */
  boool = false;
  i = 0;
  max = 1;
  while (i < m_world->getCountries().count())
  {
    if (    (RISK_GetOwnerOfCountry(i) == this)
          && (RISK_GetNumArmiesOfCountry(i) > max)
          && !GAME_IsEnemyAdjacent(i))
    {
      max = RISK_GetNumArmiesOfCountry(i);
      iCountry = i;
      boool = true;
    }
    i++;
  }
  if (boool)
  {
    destCountry = GAME_FindEnemyAdjacent(iCountry);
    if (destCountry >= 0)
    {
      AI_Move (iCountry, destCountry,
                  RISK_GetNumArmiesOfCountry(iCountry)-1);
      return true;
    }
    else
        boool = false;
  }

  /* Try to move an unused army in a country witch have a frontier */
  iCountry = 0;
  while (!boool && (iCountry < m_world->getCountries().count()))
  {
    if (    (RISK_GetOwnerOfCountry(iCountry) == this)
          && (RISK_GetNumArmiesOfCountry(iCountry) > 2)
          && !GAME_IsEnemyAdjacent(iCountry))
    {
      destCountry = GAME_FindEnemyAdjacent(iCountry);
      if (destCountry >= 0)
      {
        AI_Move (iCountry, destCountry,
                  RISK_GetNumArmiesOfCountry(iCountry)-1);
        return true;
        boool = true;
      }
    }
    iCountry++;
  }

  iCountry = 0;
  while (!boool && (iCountry < m_world->getCountries().count()))
  {
    if (    (RISK_GetOwnerOfCountry(iCountry) == this)
          && (RISK_GetNumArmiesOfCountry(iCountry) > 2)
          && !GAME_IsEnemyAdjacent(iCountry))
    {
      i = 0;
      while (!boool && (i < 6) && (RISK_GetAdjCountryOfCountry(iCountry, i) != -1))
      {
        destCountry = RISK_GetAdjCountryOfCountry(iCountry, i);
        if (RISK_GetNumArmiesOfCountry(destCountry) == 1)
        {
          AI_Move (iCountry, destCountry,
                    RISK_GetNumArmiesOfCountry(iCountry)/2);
          return true;
          boool = true;
        }
        if (!boool)
            i++;
      }
    }
    iCountry++;
  }
  return false;
}

// void AIColsonPlayer::ExchangeCards(const Player* player)
// {
//   int i, j, nb, typ, piCards[4], nbCards[4], piCardValues[MAX_CARDS];
//   bool fOptimal, fStronger, fSmaller;
// 
//   fStronger = isStrongerPlayer(player);
//   fSmaller = isSmallerPlayer(player);
//   nb = RISK_GetNumCardsOfPlayer(player);
//   do
//     {
//       piCards[0]=piCards[1]=piCards[2]=piCards[3]=-1;
//       nbCards[0]=nbCards[1]=nbCards[2]=nbCards[3]=0;
//       fOptimal = false;
//       for (i=0; i<nb; i++)
//         {
//           piCardValues[i] = RISK_GetCardOfPlayer(player, i);
//           /* Set the type of the card */
//           if (piCardValues[i]<m_world->getCountries().count())
//             {
//               typ = piCardValues[i] % 3;
// 	      nbCards[typ]++;
//               if (RISK_GetOwnerOfCountry(piCardValues[i]) == player)
//         	  piCards[typ] = i;
//               else if (piCards[typ] == -1)
//          	  piCards[typ] = i;
//             }
// 	  else  /* Joker */
//             {
// 	      piCards[3] = i;
// 	      nbCards[3]++;
// 	    }
//         }
//       if ((nbCards[0]>0)&&(nbCards[1]>0)&&(nbCards[2]>0))
//         {
//           AI_ExchangeCards(piCards);
//           fOptimal = true;
//         }
//       else if ((nbCards[3]>=2)&&((nbCards[0]>0)||(nbCards[1]>0)||(nbCards[2]>0)))
//         {
//           if (nbCards[2]>1)
//             {
//               piCards[1] = piCards[3];
//               j = 0;
//             }
//           else if (nbCards[1]>1)
//             {
//               piCards[0] = piCards[3];
//               j = 2;
//             }
//           else if (nbCards[0]>1)
//             {
//               piCards[2] = piCards[3];
//               j = 1;
//             }
//           else if (nbCards[2]>0)
//             {
//               piCards[1] = piCards[3];
//               j = 0;
//             }
//           else if (nbCards[1]>0)
//             {
//               piCards[0] = piCards[3];
//               j = 2;
//             }
//           else
//             {
//               piCards[2] = piCards[3];
//               j = 1;
//             }
//           i = 0;
//           while (i<nb)
//             {
//               if (piCardValues[i] >= m_world->getCountries().count())
//                 {
//                   piCards[j]=i;
//                   i = nb;
//                 }
//               else
//                   i++;
//             }
//           AI_ExchangeCards(piCards);
//           fOptimal = true;
//         }
//       else if (!fStronger && (nbCards[0]>=3))
//         {
//           j = 0;
//           for (i=0; i<nb; i++)
//               if ((piCardValues[i] % 3) == 0)
//                   piCards[j++]=i;
//           AI_ExchangeCards(piCards);
//         }
//       else if (fSmaller && (nbCards[1]>=3))
//         {
//           j = 0;
//           for (i=0; i<nb; i++)
//               if ((piCardValues[i] % 3) == 1)
//                   piCards[j++]=i;
//           AI_ExchangeCards(piCards);
//         }
//       else if (fSmaller && (nbCards[2]>=3))
//         {
//           j = 0;
//           for (i=0; i<nb; i++)
//               if ((piCardValues[i] % 3) == 2)
//                   piCards[j++]=i;
//           AI_ExchangeCards(piCards);
//         }
//       else if (nb >= 5)
//         {
//           if ((nbCards[0]>0)&&(nbCards[1]>0)&&(nbCards[3]>0))
//               piCards[2] = piCards[3];
//           else if ((nbCards[0]>0)&&(nbCards[3]>0)&&(nbCards[2]>0))
//               piCards[1] = piCards[3];
//           else if ((nbCards[3]>0)&&(nbCards[1]>0)&&(nbCards[2]>0))
//               piCards[0] = piCards[3];
//           else if (nbCards[0]>=3)
//             {
//               j = 0;
//               for (i=0; i<nb; i++)
//                   if ((piCardValues[i] % 3) == 0)
//                       piCards[j++]=i;
//             }
//           else if (nbCards[1]>=3)
//             {
//               j = 0;
//               for (i=0; i<nb; i++)
//                   if ((piCardValues[i] % 3) == 1)
//                       piCards[j++]=i;
//             }
//           else if (nbCards[2]>=3)
//             {
//               j = 0;
//               for (i=0; i<nb; i++)
//                   if ((piCardValues[i] % 3) == 2)
//                       piCards[j++]=i;
//             }
//           else
//             {
//               piCards[0] = 0;
//               piCards[1] = 1;
//               piCards[2] = 2;
//             }
//           AI_ExchangeCards(piCards);
//         }
//       nb = RISK_GetNumCardsOfPlayer(player);
//     }
//   while ((fOptimal && (nb >= 3)) || (nb >= 5));
// }

void AIColsonPlayer::COLSON_Play(void *pData, int iCommand, void *pArgs)
{
  switch(iCommand)
    {
    case AI_INIT_ONCE:
    {
      /* InitClient((int)pArgs); */
    }
    break;

    case AI_INIT_TURN:
    {
      m_numTurn[m_game->currentPlayer()]++;
    }
    break;

    case AI_FORTIFY:
    {
      Fortify();
    }
    break;

    case AI_PLACE:
    {
      Place();
    }
    break;

    case AI_ATTACK:
    {
      Attack();
    }
    break;

    case AI_MOVE_MANUAL:
    {
      HowManyArmiesToMove((int *)pArgs);
    }
    break;

    case AI_MOVE:
    {
      Move();
    }
    break;

    case AI_EXCHANGE_CARDS:
    {
//         ExchangeCards(m_game->currentPlayer());
    }
    break;

    case AI_SERVER_MESSAGE:
    {
    }
    break;

  }

//   return pData;
}

void AIColsonPlayer::finalize()
{
  kdDebug() <<"AIColsonPlayer::finalize" << endl; 
  int i, nb;
  
  for (i=0; i<m_game->playerList()->count(); i++)
  {
    m_isEnemyPlayer[(Player*)m_game->playerList()->at(i)] = 0;
  }
  PlayersArray::iterator it = m_game->playerList()->begin();
  PlayersArray::iterator it_end = m_game->playerList()->end();
  for (; it != it_end; it++)
  {
    Player* player = (Player*)*it;
    m_piContinent[player] = -1;
    m_numTurn[player] = 0;
    if (dynamic_cast<AIColsonPlayer*>(player)!=0)
        m_isEnemyPlayer[player] = 1;
    else if (player->author() == AUTHOR)
        m_isEnemyPlayer[player] = 2;
    else if (player->isAI())
        m_isEnemyPlayer[player] = 3;
    else
        m_isEnemyPlayer[player] = 4;
  }
  m_levelEnemy = 3;
  computeChoiceOfContinent();
  m_initialized = true;
  kdDebug() <<"    init done." << endl; 
}

int AIColsonPlayer::AI_Place(int iCountry, int iNumArmies)
{
  kdDebug() << "AIColsonPlayer::AI_Place " << iNumArmies 
            << " on country number " << iCountry << endl;
  m_placeData = new PlaceData();
  m_placeData->dest = m_game->game()->theWorld()->getCountryAt(iCountry);
  m_placeData->nb = iNumArmies;
  return 0;
}

int AIColsonPlayer::AI_Move(int iSrcCountry, int iDstCountry, int iNumArmies)
{
  m_src = m_game->game()->theWorld()->getCountryAt(iSrcCountry);
  m_dest = m_game->game()->theWorld()->getCountryAt(iDstCountry);
  kdDebug() << "AIColsonPlayer::AI_Move " << iNumArmies << " armies from " 
      << m_src->name() << " to " << m_dest->name() << endl;
  QByteArray buffer;
  QDataStream stream(buffer, IO_WriteOnly);
  stream <<  QString("actionMove") << m_src->centralPoint();
  aiPlayerIO()->sendInput(stream,true);
  return 0;
}

//////////////////////////////////////////////////////////
// Tool functions reimplemented from other parts of XFrisk
//////////////////////////////////////////////////////////

Player* AIColsonPlayer::RISK_GetOwnerOfCountry(int i)
{
//   kdDebug() << "Owner of country " << i << " is " << m_world->getCountries().at(i)->owner()->name() << endl; 
  return m_world->getCountries().at(i)->owner();
}

int AIColsonPlayer::RISK_GetContinentOfCountry(int i)
{
  Continent* continent = m_world->getCountries().at(i)->continent();

  for (uint cont=0; cont<m_world->getContinents().count(); cont++)
  {
    if (m_world->getContinents().at(cont) == continent)
    {
      return cont;
    }
  }

  return -1;
}


bool AIColsonPlayer::GAME_IsEnemyAdjacent(int i)
{
  Country* country = m_game->game()->theWorld()->getCountryAt(i);
  QPtrVector<Country> neighbours = country->neighbours();
  for (uint j = 0; j < neighbours.size(); j++)
  {
    if ( neighbours[j]->owner() != country->owner() )
    {
      return true;
    }
  }
  return false;
}

int AIColsonPlayer::RISK_GetNumArmiesOfCountry(int i)
{
  return m_world->getCountries().at(i)->nbArmies();
}


/** Returns the position in the countries list of the ith adjacent country of country iCountry */
int AIColsonPlayer::RISK_GetAdjCountryOfCountry(int iCountry, int j)
{
  Country* country = m_world->getCountries().at(iCountry);
  if (j >= country->neighbours().count())
  {
    return -1;
  }
  Country* neighbour = country->neighbours()[j];
  for (int i=0; i<m_world->getCountries().count(); i++)
  {
    if (m_world->getCountries().at(i) == neighbour)
    {
      return i;
    }
  }
  return -1;
}

int AIColsonPlayer::RISK_GetNumArmiesOfPlayer(Player* player)
{
  return player->getNbAvailArmies();
}

int AIColsonPlayer::RISK_GetNumCountriesOfPlayer(const Player* player)
{
  return player->countries().count();
}


} // closing namespace GameLogic

} // closing namespace Ksirk

