/***********************************************************************
 *
 *   18-8-95  Created by Jean-Claude Colson.
 *  for XFrisk
 *   RISK game player.
 *   $Id: aiColson.c,v 1.5 1999/11/07 15:57:29 tony Exp $
 *
 ***********************************************************************/

#ifndef KSIRK_GAMELOGIC_AICOLSONPLAYER_H
#define KSIRK_GAMELOGIC_AICOLSONPLAYER_H

#include "aiplayer.h"
// #include "aiClient.h"
// #include "client.h"
// #include "debug.h"
// #include "types.h"
// #include "riskgame.h"
// #include "game.h"
// #include "utils.h"

#define MAX_CARDS       10

namespace Ksirk
{

namespace GameLogic
{
class Player;
class Continent;
class GameAutomaton;

class AIColsonPlayer : public AIPlayer
{
public:
  /** 
    * Constructor with simple initializations
    */
  AIColsonPlayer(
          const QString & nomPlayer, unsigned int nbArmies,
          Nationality * myNation,  PlayersArray& players, ONU* world,
          GameAutomaton* game );

  virtual ~AIColsonPlayer();


  virtual QString author() {return "Jean-Claude COLSON";}

  /** Returns a pair of countries where the attacker have enough armies to attack and the defender is a ennemy neighbour of the attacker */
  virtual std::pair< const Country*, const Country* > chooseBelligerant();

  /**
    * Chooses the next action. In the current basic setting, chooses at random
    * between the three possibilities. For each, chooses randomly the
    * parameters.If the randomly choosen parameters end by an impossible 
    * action, continue with next player.
    */
  virtual void chooseAttackMoveArmiesOrNextPlayer();

  /**
    * Chooses a country to receive a new army in dotation
    */
  virtual Country* chooseReceivingCountry();
  
  /**
    * makes all what is necessary to prepare and start the moving of armies
    */
  virtual bool moveArmiesAction();
  
  /**
    * computes the next continent to try to conquer or defend
    */
  virtual void computeChoiceOfContinent(void);
  
  /** 
    * Called once when all players are created/loaded/joined and when the
    * game can start. Allows to initialize AIs with public data about other
    * players.
    */
  virtual void finalize();

private:

  struct PlaceData
  {
    Country* dest;
    uint nb;
  };



  Player* RISK_GetOwnerOfCountry(int i);
  int RISK_GetContinentOfCountry(int i);
  int RISK_GetNumArmiesOfCountry(int i);
  int RISK_GetNumArmiesOfPlayer(Player* player);
  /** 
    * Returns the position in the countries list of the ith adjacent country of
    * country iCountry 
    */
  int RISK_GetAdjCountryOfCountry(int iCountry, int i);
  int RISK_GetNumCountriesOfPlayer(const Player* player);
  bool GAME_IsEnemyAdjacent(int i);
  int FindEnemyAdjacent(int iCountry, int distance);
  int GAME_FindEnemyAdjacent(int country);



  void COLSON_Play(void *pData, int iCommand, void *pArgs);
  
  /* The species */
  // DefineSpecies(
  // 	      COLSON_Play,
  //               "Ordinateur",
  // 	      AUTHOR,
  // 	      "0.01",
  //               "Machine violente."
  // 	      )
  
  int getTotalArmiesOfPlayer(const Player* player);
  
  bool isContinentOfMission(const Player* player, const Continent* continent);
  
  bool isEnemyPlayer(const Player* player);
  
  bool isFriendPlayer(const Player* player);
  
  int getNumEnemy();
  
  bool isStrongerPlayer(const Player* player);
  
  bool isSmallerPlayer(const Player* player);
  
  bool isContinentOfPlayer(int continent, const Player* player);
  
  int GetContinentToFortify(int *attack);
  
  int GetContinentToConquier(int *attack);
  
  int NbEnenyAdjacent(int iCountry);
  
  int NbToEqualEnenyAdjacent(int iCountry);
  
  bool ComputerAttack(int destCountry, bool die, int dif, int iMove);
  
  bool Fortify();
  
  bool Place();
  
  bool AttackEnemy();
  
  bool Attack();
  
  void HowManyArmiesToMove(int *nb);
  
  bool Move();
  
  int AI_Place(int iCountry, int iNumArmies);
  int AI_Move(int iSrcCountry, int iDstCountry, int iNumArmies);

//   void ExchangeCards(const Player* player);

  std::map<const Player*, int> m_numTurn;
  std::map<const Player*, int> m_piContinent;
  std::map<const Player*, int> m_isEnemyPlayer;
  int m_levelEnemy;
  
  /** false while finalize has not been called */
  bool m_initialized;

  int Attack_SrcCountry;
  int Attack_DestCountry;

  PlaceData* m_placeData;
};

} // closing namespace GameLogic

} // closing namespace Ksirk

#endif
