/***************************************************************************
                          aiplayer.cpp  -  description
                             -------------------
    begin                : sam sep 14 2002
    copyright            : (C) 2002 by Gael de Chalendar
    email                : kleag@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* Local includes */
#include "aiplayer.h"
#include "aiplayerio.h"
#include "dice.h"
#include "nationality.h"
#include "onu.h"
#include "kgamewin.h"
#include "gameautomaton.h"

#include <kgame/kplayer.h>

/* Includes for QT */
#include <qptrlist.h>


/* Includes for KDE */
#include <kdebug.h>
#include <klocale.h>
#include <kmessagebox.h>


/* Includes for the STL */
#include <limits>
#include <utility>
#include <map>

#include <stdlib.h>

namespace Ksirk
{

namespace GameLogic
{

/**
  * Constructor with simple initializations
  */
AIPlayer :: AIPlayer(
        const QString & nomPlayer,
        unsigned int nbArmies,
         Nationality * myNation,
         PlayersArray& players,
         ONU* world,
         GameAutomaton* game ) :
    Player(nomPlayer, nbArmies, myNation) ,
  allPlayers(players),
  m_world(world),
  m_game(game),
  stopMe(true),
  m_src(0), m_dest(0),
  m_toMove(std::numeric_limits< unsigned int>::max()),
  m_hasVoted(false)
{
//   kdDebug() << "AIPlayer constructor" << endl;
}


/**
  * This function is called whenever the player should choose an action (
  * attack, defense, etc.). It has the responsability to choose the correct
  * action depending on the state of the game.
  */
void AIPlayer::actionChoice(GameLogic::GameAutomaton::GameState state)
{
//   kdDebug() << "AIPlayer " << name() << ": Action choice (state is " << GameLogic::GameAutomaton::single().stateName() << ")" << endl;
  
  QByteArray buffer;
  QDataStream stream(buffer, IO_WriteOnly);
  QByteArray buffer2;
  QDataStream stream2(buffer2, IO_WriteOnly);
  if ( (m_game-> currentPlayer() == this) || state == GameLogic::GameAutomaton::WAITDEFENSE
       ||  state == GameLogic::GameAutomaton::WAIT_RECYCLING)
  {
//     kdDebug() << name()  << " : choosing my action" << endl;
    switch (state)
    {
      case GameLogic::GameAutomaton::WAITDEFENSE :
        chooseDefenseAction();
        break;
      case GameLogic::GameAutomaton::WAIT_RECYCLING :
        chooseWetherToRecycle();
        break;
      case GameLogic::GameAutomaton::INVADE :
        chooseInvasionAction();
        break;
      case GameLogic::GameAutomaton::WAIT :
//         kdDebug() << "AIPlayer::actionChoice() WAIT " << name() << endl;
        chooseAttackMoveArmiesOrNextPlayer();
        break;
      case GameLogic::GameAutomaton::INIT: 
        break;
      case GameLogic::GameAutomaton::INTERLUDE: ;
      case GameLogic::GameAutomaton::NEWARMIES :
        placeArmiesAction();
        break;
      case GameLogic::GameAutomaton::ATTACK2 :
        if (m_dest != 0)
        {
          stream << QString("actionLButtonUp") << m_dest->centralPoint();
          m_src = 0;
          m_dest = 0;
          aiPlayerIO()->sendInput(stream,true);
        }
        break;
    case GameLogic::GameAutomaton::SHIFT1 :
      if (m_src != 0 && m_dest != 0)
      {
        stream << QString("actionLButtonDown") << m_src->centralPoint();
        aiPlayerIO()->sendInput(stream,true);
        stream2 << QString("actionLButtonUp") << m_dest->centralPoint();
        m_toMove = 0;
        aiPlayerIO()->sendInput(stream2,true);
      }
      break;
    case GameLogic::GameAutomaton::SHIFT2 :
      if (m_dest != 0)
      {
        chooseNbToMoveOrStop();
      }
      break;
    case GameLogic::GameAutomaton::GAME_OVER :
      stop();
      break;
    default :;
//         kdDebug() << "AIPlayer::actionChoice() OTHER STATE:" << state << " "  << name() << endl;
    }
  }
//    kdDebug() << "OUT AIPlayer::actionChoice()" << endl;
}

/**
  * Chooses the next action. In the current basic setting, chooses at random
  * between the three possibilities. For each, chooses randomly the parameters.
  * If the randomly choosen parameters end by an impossible action,  continue
  * with next player.
  */
void AIPlayer::chooseAttackMoveArmiesOrNextPlayer()
{
//     kdDebug() << "AIPlayer::chooseAttackMoveArmiesOrNextPlayer() " << endl;
    unsigned int dice = Dice::roll(12);
//     kdDebug() << "AIPlayer Dice rolled on " << dice << endl;
     switch ( dice )
    {
        case 1: ; case 2:; case 3:; case 4:;case 5: ; case 6:;case 7:;case 8:;case 9: ;case 10: ; case 11: // attack
        {
          if (!attackAction())
          {
            nextPlayerAction();
          }
        }
        break;
/*        case 11 : // next player
            nextPlayerAction();
        break;
*/
        case 12 : // move armies
            moveArmiesAction();
        break;
    }
//    kdDebug() <<"OUT AIPlayer::chooseAttackMoveArmiesOrNextPlayer()" << endl;
}

/**
  * Returns a pair of countries where the attacker have enough armies to attack
  * and the defender is a ennemy neighbour of the attacker
  */
std::pair<const Country*, const Country*> AIPlayer::chooseBelligerant()
{
    std::multimap<const Country*, const Country*> candidates;

//     kdDebug() << name() << " : AIPlayer::chooseBelligerant()" << endl;
    // Builds the list of countries of the player that have enough armies and a good neighbour
    QPtrList< Country > list = countries();
    if ( ! list.isEmpty() )
    {
        QPtrListIterator< Country > outer(list);
        const Country* candidateSource;
//        kdDebug() << name() << "  choosing belligerants, candidate sources " << endl;
        while ( ( ! outer.isEmpty()) && ( (candidateSource = outer.current()) != 0 ) )
        {
//            kdDebug() << name() << "  choosing belligerants, looking at candidate source : " << candidateSource-> name() << endl;
            // Enough armies
            if ( candidateSource-> nbArmies() > 1 )
            {
//                kdDebug() << name() << "  choosing belligerants, candidate source has enough armies." << endl;
//                kdDebug() << name() << "  choosing belligerants, candidate targets " << endl;
                for ( unsigned int inner = 0; inner < m_world-> getCountries().count(); inner++)
                {
                    const Country* candidateTarget = m_world-> getCountries().at(inner);
//                    kdDebug() << name() << "  choosing belligerants, looking at candidate target : " << candidateTarget-> name() << endl;
                    // Enemy neigbour
                    if (            ( candidateTarget != candidateSource ) 
                            &&     (candidateSource-> owner() != candidateTarget-> owner())
                            &&     (candidateSource-> communicateWith(candidateTarget) )
                            &&     !(candidateTarget->name().contains("NULL") ) )
                    {
//                        kdDebug() << name() << "  choosing belligerants, adding target / source : "
//                                << candidateSource-> name() << " / " << candidateTarget-> name() << endl;
                        candidates.insert(std::make_pair(candidateSource, candidateTarget));
                    }
                }
            }
            ++outer;
        }
        if ( candidates.size() == 0 )
        {
//            kdDebug() << name() << " OUT AIPlayer::chooseBelligerant() ; map size = 0 ; it isn't possible to attack." << endl;
            return (std::make_pair(static_cast< Country*>(0), static_cast< Country*>(0)));
        }
        uint which = Dice::roll(candidates.size()) - 1;
//        kdDebug() << "Which = " << which <<endl;
        std::multimap<const Country*, const Country*>::const_iterator it;
        unsigned int i = 0;
        for ( it = candidates.begin(); it != candidates.end() ; it++, i++ )
        {
            if (which == i )
            {
//                kdDebug() << "OUT AIPlayer::chooseBelligerant() : " << endl ;
                return (*it);
            }
        }
    }
//    kdDebug() << "OUT AIPlayer::chooseBelligerant() : do I own no country ???" << endl;
    return (std::make_pair(static_cast< Country* >(0), static_cast< Country* >(0)));
}

/**
  * Chooses a country to receive a new army in dotation. random choice in the 
  * list of the player countries that have at least one neigbour belonging to 
  * someone else
  */
Country* AIPlayer::chooseReceivingCountry()
{
//     kdDebug() << "AIPlayer::chooseReceivingCountry()" << endl;
    QPtrList<Country> myCountries = countries();
    if (myCountries.count() == 0)
    {
        kdError() << "AIPlayer::chooseReceivingCountry() EMPTY LIST" << endl;
        return 0;
    }
    QPtrList<Country> withNeighbours;
    
    for (unsigned int i = 0 ; i < myCountries.count(); i++)
    {
        if ( ( m_world-> neighboursNotBelongingTo(*(myCountries.at(i)), static_cast< const Player * >(this) ) ).count() )
            withNeighbours.append(myCountries.at(i));
    }

    int which = Dice::roll(withNeighbours.count()) - 1;
    if (which == -1)
        kdDebug() << name() << " has no enemy neighbour... should not happen." << endl;
//    kdDebug() << "\tChoosed: " << list.at(which)-> name() << endl;
    
    return withNeighbours.at(which);
//    kdDebug() << "OUT AIPlayer::chooseReceivingCountry()" << endl;
}

/** 
  * Returns true (an AIPlayer is an AI)
  */
bool AIPlayer::isAI() const
{
    return true;
}

void AIPlayer::run()
{
//    kdDebug() << name() << " : AIPlayer::run()" << endl;
  stopMe = false;
  while ( ! stopMe )
  {
    actionChoice(GameLogic::GameAutomaton::single().state());
    msleep( 500 );
  }
//    kdDebug() << "OUT AIPlayer::run()" << endl;
}

/** set stopMe to true in order for the run method to return */
void AIPlayer::stop()
{
    stopMe = true;
}

/**
  * make all what is necessary to prepare and launch an attack
  */
bool AIPlayer::attackAction() 
{
//   kdDebug() << "AIPlayer::attackAction" << endl;
  QByteArray buffer;
  QDataStream stream(buffer, IO_WriteOnly);
  std::pair<const Country* , const Country* > srcDest = chooseBelligerant();
  if ( (srcDest.first == 0) || (srcDest.second == 0) )
  {
//     kdDebug() << "AIPlayer::attackAction: no attack available" << endl;
//       nextPlayerAction(); 
      return false;
  }
  m_src = srcDest.first;
  m_dest = srcDest.second;
  uint srcNbArmies = m_src->nbArmies();
  kdDebug() << name()  << " : ATTACK" << endl;
  kdDebug() << "    " << name()  << " : attacks from "  << m_src-> name() 
                  << " (" << srcNbArmies << " armies)" << endl;
  kdDebug() << "    " << name()  << " : attacks " << m_dest-> name() << endl;
      
  uint nbAttack = 0;
  if (srcNbArmies == 1)
  {
    kdError() << "AI player " << name() << " country " << m_src->nbArmies() << "have only one army. Should not be choosed to attack." << endl;
    exit();
  }
  if (srcNbArmies >= 2) {nbAttack = 1;}
  if (srcNbArmies >= 3) {nbAttack = 2;}
  if (srcNbArmies >= 4) {nbAttack = 3;}
  m_nbAttack = nbAttack;
  kdDebug() << "    " << name()  << " : attacks with " << nbAttack << " armies." << endl;
  QPoint point;
  switch (nbAttack)
  {
    case 1: 
    stream << QString("actionAttack1") << point;
    break;
    case 2: 
    stream << QString("actionAttack2") << point;
    break;
    case 3: 
    stream << QString("actionAttack3") << point;
    break;
    default:
      kdError() << "The attacker tries to attack with a number of armies different of 1, 2 or 3: that's impossible!" << endl;
      exit();      
  }
  aiPlayerIO()->sendInput(stream,true);
  QByteArray buffer2;
  QDataStream stream2(buffer2, IO_WriteOnly);
  stream2 << QString("actionLButtonDown") << m_src->centralPoint();
  aiPlayerIO()->sendInput(stream2,true);
  kdDebug() << "AIPlayer " << name()  << " : attackAction : "  << m_src-> name() << " " << m_dest-> name()
    << " " << nbAttack << endl;
  return true;
}

/**
  * makes all what is necessary to prepare and start the moving of armies
  */
bool AIPlayer::moveArmiesAction() 
{
  kdDebug() << "AIPlayer::moveArmiesAction" << endl;
  QPtrList<Country> srcList( countries() );
  if (srcList.count() == 0) 
  {
    nextPlayerAction(); 
    return false;
  }
  uint which = Dice::roll(srcList.count()) - 1;
  Country* osrc =  srcList.at(which);
//    kdDebug() << "AIPlayer::chooseAttackMoveArmiesOrNextPlayer() MOVEARMIES 1" << endl;
  if (osrc-> nbArmies() <= 1) 
  {
    nextPlayerAction();
    return false;
  }
//    kdDebug() << "AIPlayer::chooseAttackMoveArmiesOrNextPlayer() MOVEARMIES 2" << endl;
  QPtrList<Country> destList( m_world-> neighboursBelongingTo(*osrc, this) );
//    kdDebug() << "AIPlayer::chooseAttackMoveArmiesOrNextPlayer() MOVEARMIES 3" << endl;
  if (destList.count() == 0) 
  {
    nextPlayerAction();
    return false;
  }
  which = Dice::roll(destList.count()) - 1 ;
//    kdDebug() << "AIPlayer::chooseAttackMoveArmiesOrNextPlayer() MOVEARMIES 4" << endl;
  Country* odest =  destList.at(which);
  m_src = osrc;
  m_dest = odest;
  kdDebug() << "AIPlayer::chooseAttackMoveArmiesOrNextPlayer " << endl;

  QByteArray buffer;
  QDataStream stream(buffer, IO_WriteOnly);
  stream <<  QString("actionMove") << osrc->centralPoint();
  aiPlayerIO()->sendInput(stream,true);
  return true;
}

/**
 * chooses a country where to place a new army
 */
void AIPlayer::placeArmiesAction() 
{
  kdDebug() << "AIPlayer::placeArmiesAction " << name() << " ; nb to place: " << getNbAvailArmies() << endl;
  if (getNbAvailArmies() > 0)
  {
    m_hasVoted = false;
    const Country* receiver = chooseReceivingCountry();
    if (receiver == 0)
    {
      QString msg;
      QTextOStream(&msg) << i18n("Error - No receiving country selected while computer player ") << name() << i18n(" had still ") << getNbAvailArmies() << i18n(" armies to place. This is bug probably #2232 at www.gna.org.");
      KMessageBox::error(0, msg, i18n("Fatal Error"));
      exit();    
    }
    kdDebug() << "Placing an army in " << receiver->name() << endl;
    QByteArray buffer;
    QDataStream stream(buffer, IO_WriteOnly);
    stream << QString("actionLButtonDown") << receiver->centralPoint();
    aiPlayerIO()->sendInput(stream,true);
  }
  else if (GameAutomaton::changeable().state() != GameAutomaton::INTERLUDE)
  {
    kdDebug() << "No more armies to place: next player" << endl;
    stop();
    QPoint point;
    GameAutomaton::changeable().event("actionNextPlayer", point);
  }
  else if (m_game->allLocalPlayersComputer())
  {
    if (!m_hasVoted)
    {
      QByteArray buffer;
      QDataStream stream(buffer, IO_WriteOnly);
      QPoint point;
      stream << QString("slotRecyclingFinished") << point;
      aiPlayerIO()->sendInput(stream,true);
      m_hasVoted = true;
    }
  }
  
}

/**
 * Takes the decision to recycle armies or not
 */
void AIPlayer::chooseWetherToRecycle() 
{
  kdDebug() << "AIPlayer::chooseWetherToRecycle" << name() << endl;
  if (m_game->allLocalPlayersComputer())
  {
    if (!m_hasVoted)
    {
      kdDebug() << "Voting for end of recycling" << endl;
      QByteArray buffer;
      QDataStream stream(buffer, IO_WriteOnly);
      QPoint point;
      stream << QString("slotRecyclingFinished") << point;
      aiPlayerIO()->sendInput(stream,true);
      m_hasVoted = true;
    }
    else
    {
      kdDebug() << "Has already voted" << endl;
    }
  }
  else
  {
    kdDebug() << "There is local non computer players; let them vote." << endl;
  }
  // reevaluate the target continent
  computeChoiceOfContinent();
  stop();
}

/**
 * chooses to continue invasion with a certain amount of armies or to stop it
 */
void AIPlayer::chooseInvasionAction()
{
  QByteArray buffer;
  QDataStream stream(buffer, IO_WriteOnly);
  kdDebug() << QString("AIPlayer::chooseInvasionAction") << endl;
  int nbArmiesToMove =Dice::roll(m_game->game()-> getAttackingCountry()-> nbArmies()) - 1;
  QPoint point;
  while (nbArmiesToMove >= 10) 
  {
      stop(); 
    stream << QString("actionInvade10") << point;
      nbArmiesToMove -= 10;
  }
  while (nbArmiesToMove >= 5) 
  { 
      stop();
    stream << QString("actionInvade5") << point;
      nbArmiesToMove -= 5;
  }
  while (nbArmiesToMove >= 1) 
  { 
      stop();
    stream << QString("actionInvade1") << point;
    nbArmiesToMove--;
  }
  stream << QString("actionInvasionFinished") << point;
  stop();
  aiPlayerIO()->sendInput(stream,true);
}

/**
  * chooses wether to defend with one or two armies. Always chooses the maximum possible
  */
void AIPlayer::chooseDefenseAction()
{
  QByteArray buffer;
  QDataStream stream(buffer, IO_WriteOnly);
  QPoint point;
  if (m_game-> currentPlayer() == this)
    kdDebug() << "AIPlayer::chooseDefenseAction waiting defense of another one; nothing to do." << endl;
  else
  {
    kdDebug() << "AIPlayer::chooseDefenseAction " << name() << endl;
    switch (m_game-> currentPlayer()-> getNbAttack())
    {
      case 1:
      stream << QString("slotDefense1") << point;
//         m_game->slotDefense1();
      break;
      case 2: ; case 3:
        if (m_game-> game()-> getAttackedCountry()-> nbArmies() > 1)
          stream << QString("slotDefense2") << point;
//       m_game->slotDefense2();
        else
          stream << QString("slotDefense1") << point;
//       m_game->slotDefense1();
      break;
      default:
        kdError() << "The attacker attacks with a number of armies different of 1, 2 or 3: that's impossible!" << endl;
        exit();
    }
    stop();
    aiPlayerIO()->sendInput(stream,true);
  }
}

/**
 * makes what is necessary to finish my turn
 */
void AIPlayer::nextPlayerAction()
{
  kdDebug() << "AIPlayer::nextPlayerAction" << endl;
  QPoint point;
  QByteArray buffer;
  QDataStream stream(buffer, IO_WriteOnly);
  stream << QString("actionNextPlayer") << point;
  aiPlayerIO()->sendInput(stream,true);
  stop();
}

void AIPlayer::saveXml(std::ostream& xmlStream)
{
  xmlStream << "<player ai=\"true\" ";
  innerSaveXml(xmlStream);
  xmlStream << " />" << std::endl;
}

/** 
 * Makes the choice of nb armies to move during an invasion or an end of turn
 * moving 
 */
void AIPlayer::chooseNbToMoveOrStop()
{
  QByteArray buffer;
  QDataStream stream(buffer, IO_WriteOnly);
  QPoint point;
  if (m_toMove == std::numeric_limits< unsigned int>::max())
  {
    m_toMove = Dice::roll(m_src->nbArmies() - 1);
  }
  if (m_toMove >= 10)
  {
    stream << QString("actionInvade10") << point;
    m_toMove -= 10;
  }
  else if (m_toMove >= 5)
  {
    stream << QString("actionInvade5") << point;
    m_toMove -= 5;
  }
  else if (m_toMove >= 1)
  {
    stream << QString("actionInvade1") << point;
    m_toMove -= 1;
  }
  else // m_toMove == 0
  {
    stream << QString("actionInvasionFinished") << point;
    m_toMove = std::numeric_limits< unsigned int>::max();
    stop();
  }
  aiPlayerIO()->sendInput(stream,true);
}

AIPlayerIO* AIPlayer::aiPlayerIO()
{
  KGameIOList* iolist = ioList();
  for ( uint i = 0; i < iolist->count(); ++i )
    if ( iolist->at(i) && iolist->at(i)->rtti() == AIPLAYERIO )
      return dynamic_cast<AIPlayerIO*>(iolist->at(i));
  
  return 0;
}

} // closing namespace GameLogic
} // closing namespace Ksirk

#include "aiplayer.moc"
