/***************************************************************************
                          continent.cpp  -  description
                             -------------------
    begin                : sam sep 7 2002
    copyright            : (C) 2002 by Gael de Chalendar
    email                : kleag@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "continent.h"

#include "kdebug.h"

namespace Ksirk
{

namespace GameLogic
{

/** The constructor-initializer */
Continent::Continent (const QString &myName, QPtrList<Country>& myCountries, int myBonus,
                     unsigned int id) :
  members(myCountries), m_name(myName), bonus(myBonus), m_id(id)
{
  for ( uint i = 0; i < myCountries.count(); ++i )
  {    
    myCountries.at(i)->setContinent(this);
  }
}

Continent::Continent():bonus(0)
{
}

Continent::~Continent()
{
}

/** Read property of QList<Country> members. */
const QList<Country>& Continent::getMembers()
{
    return members;
}

/** Read property of QString name. */
const QString& Continent::name() const
{
    return m_name;
}

/** Read property of int bonus. */
const int& Continent::getBonus()
{
    return bonus;
}

/**
  * Returns the player that owns all the countries of this continent. 0 if none
  */
const Player* Continent::owner() const
{
//    kdDebug() << "Continent::owner for "  << m_name;
    /** The owner of the first country is the owner if there is any one*/
  const Country* firstOne = const_cast< QPtrList<Country>* >(&members)->first();
    const Player* owner = firstOne-> owner();
//    kdDebug() << "\t"  << firstOne-> name()  << " is owned by "  << owner-> name();

  for (Country* c = const_cast< QPtrList<Country>* >(&members)->next(); c != 0; c = const_cast< QPtrList<Country>* >(&members)->next())
    {
//        kdDebug() << "\t"  << c-> name() << " is owned by "  << c-> owner()-> name();
        /** if the owner of the current country is not the owner of th first
          * one, then there is two different owners and the function should
          * return 0
          */
        if (c-> owner() != owner)
        {
//            kdDebug() << "Nobody owns " << m_name << endl;
            return 0;
        }
    }
    /** There is only one owner for all hte countries ; lets return it */
//    kdDebug() << "The owner of " << m_name << " is "  << owner-> name();
    return owner;
}

void Continent::saveXml(std::ostream& xmlStream)
{
  QString name = m_name.utf8();
  name = name.replace("&","&amp;");
  name = name.replace("<","&lt;");
  name = name.replace(">","&gt;");
  xmlStream << "<continent name=\""<<name<<"\" bonus=\""<<bonus<<"\" >" << std::endl;
    QPtrListIterator< Country > membersIt(members);
    do 
    {
      (*membersIt)->saveXml(xmlStream);
      membersIt += 1;
    } while (!(membersIt.atLast()));

    xmlStream << "</continent>" << std::endl;
}

} // closing namespace GameLogic
} // closing namespace Ksirk
